{DelphiSim by Roeland van der Spek           }
{spekw@acm.org                               }
{(C) 1997                                    }
{v.2.0                                       }
{Example 1                                   }

{
  This example shows a simple simulation model of a generator process that
  cyclicly generates loads, and a machine that processes these loads. Loads
  are generated using an exponential distribution (RandIverval) and the
  bandling time at the machine is specified by another exponential distribution
  (RandProcessing). When a load has finished processing, it is destroyd.

  The simulation controll group box contains four buttons which can be used
  to controll the simulation. They demostrate the StartSimulation, ResumeSimulation
  SuspendSimulation and TerminateSimulation methods.

  If aninamtion is enabled, it will show the number of jobs waiting at the machine.
  The Animation is updated in the machine's OnArrival Event. The Code for the animation
  is situated in the machine's SimulationUpdate Event. The speed of the simulation
  can be controlled by the Simulations Animation Interval property. This property
  can be manipulated with Trackbar in the simulation settings box.

  The settings of the actual simulation model can be set using the slidebars in
  the Model Settings box. One changes the mean of the inter arrival time and
  the other changes the mean of the processing time.

  The following statistics are collected: Machine Utilization, The avarage time
  spent in the waiting queue, the average time spent on processing and the
  number of items processed by the machine. Most of these statistics are collected
  with two Tally components: TimeInQueue Tally and ProcessingTimeTally. Measurments
  are done during the machine's BeforeProcessing Events and AfterProcessing Events.
}

unit example1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Gauges, StdCtrls, ComCtrls, Spin, machine, delphiSim, generator, Random,
  Tally;

type
  TForm1 = class(TForm)
    mysim: TSimulation;
    RandInterval: TExponential;
    myStream: TRandomStream;
    GeneratorProcess1: TGeneratorProcess;
    myMachine: TMachineProcess;
    RandProcessing: TExponential;
    GroupBox1: TGroupBox;
    BtSuspend: TButton;
    BtTerminate: TButton;
    BtStart: TButton;
    BtResume: TButton;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    SEreps: TSpinEdit;
    TrackBar1: TTrackBar;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    GroupBox3: TGroupBox;
    TrackBar2: TTrackBar;
    Label6: TLabel;
    Label7: TLabel;
    TrackBar3: TTrackBar;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    GroupBox4: TGroupBox;
    Gauge1: TGauge;
    Label12: TLabel;
    LJobsPending: TLabel;
    Label14: TLabel;
    LSimtime: TLabel;
    GroupBox5: TGroupBox;
    TimeInQueueTally: TTally;
    ProcessingTimeTally: TTally;
    Animated: TCheckBox;
    Label1: TLabel;
    Label13: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    LUtilization: TLabel;
    Lprocessed: TLabel;
    LqueueTime: TLabel;
    LProcessTime: TLabel;
    procedure BtStartClick(Sender: TObject);
    procedure GeneratorProcess1Generate(Generator: TGeneratorProcess);
    procedure myMachineAfterProcessing(Load: TObject;
      machine: TMachineProcess);
    procedure myMachineAnimationUpdate(Process: TSimprocess);
    procedure myMachinearrival(Load: TObject; Machine: TMachineProcess);
    procedure TrackBar3Change(Sender: TObject);
    procedure TrackBar2Change(Sender: TObject);
    procedure mysimSimEnd(simulation: TSimulation);
    procedure TrackBar1Change(Sender: TObject);
    procedure mysimSimInit(simulation: TSimulation);
    procedure AnimatedClick(Sender: TObject);
    procedure SErepsChange(Sender: TObject);
    procedure BtTerminateClick(Sender: TObject);
    procedure BtSuspendClick(Sender: TObject);
    procedure BtResumeClick(Sender: TObject);
    procedure mysimSuspend(simulation: TSimulation);
    procedure mysimReplication(Simulation: TSimulation);
    procedure myMachineBeforeProcessing(Load: TObject;
      machine: TMachineProcess);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TLoad = class(TObject) {definition of our load type}
  public
    TimeIn:Double;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.BtStartClick(Sender: TObject);
begin
  BtStart.enabled:=false;   {set the buttons right}
  BtSuspend.enabled:=true;
  BtResume.enabled:=false;
  BtTerminate.enabled:=true;
  SEReps.enabled:=false;
  Mysim.startSimulation;    {start the simulation}
end;

procedure TForm1.GeneratorProcess1Generate(Generator: TGeneratorProcess);
var P:Tload;
begin
  P:=Tload.create;                           {generate the load}
  mymachine.process(P,RandProcessing.draw);  {send it to the machine for processing}
end;

procedure TForm1.myMachineAfterProcessing(Load: TObject;
  machine: TMachineProcess);
var P:TLoad;
begin
  P:=Load as TLoad;
  ProcessingTimeTally.Tally(Machine.simtime-P.TimeIn); {measure the processing time}
  P.free;                                              {remove the load}
end;

procedure TForm1.myMachineAnimationUpdate(Process: TSimprocess);
var M:TMachineProcess;
begin
  M:= process as TMachineProcess;
  LSimtime.caption:=format('%f',[M.simtime]);          {display the simulation time}
  If M.JobsPending > 800 then Gauge1.forecolor:=Clred  {if the number of waiting items becoms critcal,}
  else Gauge1.forecolor:=Clgreen;                      {change to red, else use a green color}
  LJobsPending.caption:=format('%d',[M.Jobspending]);  {Display the number of waiting loads}
  Gauge1.progress:= M.Jobspending;                     {Display the same number graphically}
end;

procedure TForm1.myMachinearrival(Load: TObject; Machine: TMachineProcess);
begin
  machine.UpdateAnimation;                 {when a new item arrives at the machine, the simualation is updated}
  (Load as TLoad).timeIn:=Machine.SimTime;
end;

procedure TForm1.TrackBar3Change(Sender: TObject);
begin
  RandProcessing.mean:= Trackbar3.Position;
end;

procedure TForm1.TrackBar2Change(Sender: TObject);
begin
  RandInterval.mean:=Trackbar2.position;
end;

procedure TForm1.mysimSimEnd(simulation: TSimulation);
begin
  btStart.enabled:=true;      {on simulation end, set the buttons back to default}
  btResume.enabled:=false;
  btTerminate.enabled:=false;
  btSuspend.enabled:=false;
  SEReps.enabled:=true;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  mysim.AnimationInterval:=Trackbar1.position;
end;

procedure TForm1.mysimSimInit(simulation: TSimulation);
begin

  {simulation initialization comes here}

  Simulation.AnimationInterval:=Trackbar1.position;
  RandInterval.Mean:=Trackbar2.position;
  RandProcessing.mean:=Trackbar3.position;
  TimeInQueueTally.Reset;                   {reset the tally components}
  ProcessingTimeTally.Reset;
  Simulation.animated:=Animated.checked;    {set the animation property}
end;

procedure TForm1.AnimatedClick(Sender: TObject);
begin
  Mysim.animated:=Animated.checked;
end;

procedure TForm1.SErepsChange(Sender: TObject);
begin
  mysim.repSize:=SEreps.value;
end;

procedure TForm1.BtTerminateClick(Sender: TObject);
begin
  BtTerminate.enabled:=false;
  BtStart.enabled:=False;
  BtSuspend.enabled:=false;
  BtResume.enabled:=false;
  mysim.TerminateSimulation;  {Terminate the simulation}
end;

procedure TForm1.BtSuspendClick(Sender: TObject);
begin
  BtSuspend.enabled:=false;
  BtResume.enabled:=true;
  BtTerminate.enabled:=true;
  BtStart.Enabled:=false;
  mysim.SuspendSimulation; {suspend the simulation}
end;

procedure TForm1.BtResumeClick(Sender: TObject);
begin
  BtStart.enabled:=false;
  BtSuspend.enabled:=true;
  BtResume.enabled:=false;
  BtTerminate.enabled:=true;
  MySim.resumeSimulation;    {resume the simulation}
end;

procedure TForm1.mysimSuspend(simulation: TSimulation);
begin
  BtSuspend.enabled:=false; {on simulation suspend reset buttons}
  BtResume.enabled:=true;
  BtStart.enabled:=false;
  BtTerminate.enabled:=true;
end;

procedure TForm1.mysimReplication(Simulation: TSimulation);
var percentage:double;
begin

  {at the end of each replication, the simulation results are displayed}

  percentage:=100*Mymachine.TimeActive/Simulation.repsize;
  LUtilization.caption:=format('%f%%',[percentage]);
  LProcessed.caption:=format('%d',[ProcessingTimeTally.count]);
  LQueueTime.caption:=format('%f',[TimeInQueueTally.avg]);
  LProcessTime.caption:=format('%f',[ProcessingTimeTally.avg]);
end;


procedure TForm1.myMachineBeforeProcessing(Load: TObject;
  machine: TMachineProcess);
begin
 TimeInQueueTally.tally(Machine.Simtime-(load As TLoad).TimeIn);
 (Load as Tload).timeIn:=Machine.Simtime;
end;

end.
