{DelphiSim by Roeland van der Spek           }
{spekw@acm.org                               }
{(C) 1997                                    }
{v.2.0                                       }
{Example 2                                   }

{
   This Example is a model of a machine that handles loads that
   arrive 24 hour a day at continuous intervals of 10 minutes.
   The loads are first send to a waiting process to dry. This will
   take anything between 1 and 10 minutes. Then the loads are send
   to the machine
   It takes the machine anyting between 1 and 8 minutes to complete
   processing the load. We use a TCustom Process as a simple
   generator for arriving loads.

   The machine requires an operator. This operator works normal
   working hours (beteen 8 am and 5:30 pm) and has a luchbreak
   between noon and 12:30. When the operator is absent, the machine
   wont work and the jobs will start accumulating. The worker is
   modelled by a TCustomProcess (WorkerProcess) and the synchronization
   between machine and the worker is done with a semaphore.

   When a machine starts working on a job, it will wait until the
   semaphore becomes available. After the job is completed, the
   semaphore will be released. The worker will take the semaphore
   when it goes away and releases it when he returns.

   We three Custom processes to model the clock. (this is a complet
   waste of system resources but I want to make a point :) )
   One process counts the days, another the hours and the last
   custom process counts the minutess.

   The simulation will run for 2 days, you can increase this by
   increasing the RepSize property of the Simulation. Animation
   speed can be increased by changing the simulations AnimationInterva
   property

}
unit example2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  machine, waitprocess, delphiSim, synchronization, Random, StdCtrls,
  Gauges;

type
  TForm1 = class(TForm)
    MyStream: TRandomStream;
    mysim: TSimulation;
    worker: TSimSemaphore;
    WorkGenerator: TCustomProcess;
    MyWait: TWaitProcess;
    mymachine: TMachineProcess;
    CustomProcess2: TCustomProcess;
    LDay: TLabel;
    WaitTime: TTriangular;
    JobsWaiting: TGauge;
    LSimtime: TLabel;
    LJobsDone: TLabel;
    Button1: TButton;
    LWorkStatus: TLabel;
    dayproc: TCustomProcess;
    hourproc: TCustomProcess;
    minproc: TCustomProcess;
    LJobsPending: TLabel;
    procedure WorkGeneratorProcessBody(Process: TSimprocess);
    procedure CustomProcess2AnimationUpdate(Process: TSimprocess);
    procedure CustomProcess2ProcessBody(Process: TSimprocess);
    procedure MyWaitContinue(Load: TObject);
    procedure mymachineAfterProcessing(Load: TObject;
      machine: TMachineProcess);
    procedure mymachinearrival(Load: TObject; Machine: TMachineProcess);
    procedure mymachineAnimationUpdate(Process: TSimprocess);
    procedure mysimSimEnd(simulation: TSimulation);
    procedure Button1Click(Sender: TObject);
    procedure mysimSimInit(simulation: TSimulation);
    procedure mymachineBeforeProcessing(Load: TObject;
      machine: TMachineProcess);
    procedure dayprocProcessBody(Process: TSimprocess);
    procedure minprocProcessBody(Process: TSimprocess);
    procedure minprocAnimationUpdate(Process: TSimprocess);
    procedure hourprocProcessBody(Process: TSimprocess);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  day:Integer;
  workcount:Integer;
  workstatus:String;
  min:Integer;
  hour:integer;

implementation

{$R *.DFM}

{simple custom made generator}
procedure TForm1.WorkGeneratorProcessBody(Process: TSimprocess);
var P:TObject;
begin
  with process do
  begin
    while not false do              {do forever}
    begin
      P:=TObject.create;            {create a load}
      MyWait.wait(P,waittime.draw); {send it to the machine}
      Holdfor(10);                  {wait for 10 minutes and start again}
    end;
  end;
end;

procedure TForm1.CustomProcess2AnimationUpdate(Process: TSimprocess);
begin
  LworkStatus.caption:='Worker is '+workstatus; {update the worker status}
end;

procedure TForm1.CustomProcess2ProcessBody(Process: TSimprocess);
begin
  with process do
  begin
    Day:=0;
    updateanimation;
    while not false do
    begin
      Holdfor(8*60); {the workday starts at 8:00 am}
      workStatus:='working';
      updateanimation;        {tell the mainthread to update the animation}
      worker.release(1);

      Holdfor(4*60); {The worker has lunch at noon }
      waitfor(worker);
      WorkStatus:='having lunch';
      updateanimation;
      holdfor(30);  {lunch takes 30 minutes}
      worker.release(1);
      workStatus:='working';
      updateanimation;

      Holdfor(5*60); {the workday ends at 5:30 pm}
      WorkStatus:='at home';
      updateanimation;
      waitfor(worker);
      holdfor(6.5*60);
      updateanimation;
    end;
  end;
end;

procedure TForm1.MyWaitContinue(Load: TObject);
begin
  Mymachine.process(load,DrawUniform(1,8,mystream));  {stop waiting and start processing}
end;

procedure TForm1.mymachineAfterProcessing(Load: TObject;
  machine: TMachineProcess);
begin
  machine.UpdateAnimation; {after processing we update the animation}
  Inc(workCount);          {we increase the work count}
  Load.free;               {we destroy the load...}
  worker.release(1);       {and release the worker}
end;

procedure TForm1.mymachinearrival(Load: TObject; Machine: TMachineProcess);
begin
  Machine.updateAnimation;  {when a load arrives we update the machine animation}
end;

procedure TForm1.mymachineAnimationUpdate(Process: TSimprocess);
var m:TMachineProcess;
begin
  m:=Process as TMachineProcess;
  JobsWaiting.progress:=M.JobsPending;      {show the pending jobs}
  LJobsPending.caption:=format('Pending Jobs: %d',[m.JobsPending]);
  LJobsDone.caption:=format('Jobs Done: %d',[workcount]); {show the number of completed jobs}
end;

procedure TForm1.mysimSimEnd(simulation: TSimulation);
begin
  button1.enabled:=true; {after the simulation has completed the startbutton is enabled}
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  button1.enabled:=false;
  mysim.startSimulation;  {start the simulation}
end;

procedure TForm1.mysimSimInit(simulation: TSimulation);
begin
  worker.clear;         {clear the semaphore}
  worker.available:=0;  {set the semaphore value}
  workcount:=0;         {etc.}
  workstatus:='at home';
end;

procedure TForm1.mymachineBeforeProcessing(Load: TObject;
  machine: TMachineProcess);
begin
  Machine.waitfor(worker);    {Before processing, wait for the worker}
end;

procedure TForm1.dayprocProcessBody(Process: TSimprocess);
begin
  day:=0;
  with process do
  begin
   while not false do
   begin
     day:=trunc((process.simtime+0.01) / 1440);
     holdfor(1440);
   end
  end;
end;

procedure TForm1.minprocProcessBody(Process: TSimprocess);
begin
  min:=0;
  with process do
  begin
    while not false do
    begin
      min:= trunc(simtime) mod 60;
      updateanimation;
      holdfor(1);
    end;
  end;
end;

procedure TForm1.minprocAnimationUpdate(Process: TSimprocess);
begin
  LSimtime.caption:= format('Simtime: %2d:%2d',[hour,min]);
  LDay.caption:=format('Day: %d',[day+1]);
end;

procedure TForm1.hourprocProcessBody(Process: TSimprocess);
begin
 hour:=0;
  with process do
  begin
    while not false do
    begin
      hour:= (trunc((simtime+0.01) / 60) mod 24);
      holdfor(60);
    end;
  end;
end;

end.
